%    example2a.m:  Write some application data to a SmartDongle.

%    www.smartdongle.com

%    MicroWorks, Inc.
%    2808 North Cole Road
%    Boise, ID 83704
%    (208)-375-1234

function example2a()
    % Your personal keys go here.
    P1 = hex2uint64('EC6CC589AEFD1E75');
    P2 = hex2uint64('FCEC0A6A82747B3F');
    
    % User space memory address to be written to (0-32K)
    address = 0;
    
    % Data to be written to the SmartDongle. Can be anything.
    WriteData = 'This is data written to the SmartDongle, it can be serial numbers, program parameters, etc.  Note that this data can be read by observing activity on the USB bus. Refer to Example3a.cs and Example3b.cs for example of encrypting your application data.';
    
    % All data to be written to the SmartDongle must be in a byte array.
    % This converts the above string into a byte array.
    WriteBytes = unicode2native(WriteData, 'US-ASCII');
    
    % Load the USK library for SmartDongle functions.
    loadlibrary('USK', 'usk.h');
    
    % Call LedRed from the USK library.
    calllib('USK','LedRed');
    
    % Create a c-style pointer to a uint8Ptr object containing the data to
    % be written to the SmartDongle.
    Buffer = libpointer('uint8Ptr', WriteBytes);
    
    % Call Write from the USK library to write the data contained in
    % WriteBytes to the SmartDongle. Pass P1, P2, memory address to write
    % to, pointer to the data, and length of the data to be written.
    % Returns and error code and and the data that was written.
    [error, Buffer] = calllib('USK', 'Write', P1, P2, address, Buffer, length(WriteData) );
    
    % An error code of zero indicates a successful write, all other numbers
    % are errors.
    if(error ~= 0)
        % Create a pointer to a c-style string with 128 characters to allow
        % space for the max error message length.
        ErrMsg = libpointer('cstring', blanks(128));
        
        % Call GetErrorString from the USK library passing in the c-style
        % string ErrMsg and the error code from the previous Read call. The
        % error message will be returned into ErrMsg.
        [ErrMsg] = calllib('USK', 'GetErrorString', ErrMsg, error);
        
        % Print the error message.
        fprintf('\nFailed: Could not write application data to SmartDongle\n%s\n\n',ErrMsg)
    else
        % Print success.
        fprintf('\nSuccess: Wrote application data to SmartDongle\n\n')
    end
    
    % Call LedGreen from the USK library.
    calllib('USK', 'LedGreen');
    
    % Unload the USK library.
    unloadlibrary('USK');


end

