%    example2b.m: Read application data written to SmartDongle in example2a.c

%    www.smartdongle.com

%    MicroWorks, Inc.
%    2808 North Cole Road
%    Boise, ID 83704
%    (208)-375-1234

function example2b()
    % Your personal keys go here.
    P1 = hex2uint64('EC6CC589AEFD1E75');
    P2 = hex2uint64('FCEC0A6A82747B3F');
    
    % User space memory address to read from on the SmartDongle (0-32K).
    address = 0;
    
    % Data written to the SmartDongle in example2a. Used here to compare
    % and verify that what was written is what was read back.
    WriteData = 'This is data written to the SmartDongle, it can be serial numbers, program parameters, etc.  Note that this data can be read by observing activity on the USB bus. Refer to Example3a.cs and Example3b.cs for example of encrypting your application data.';
    
    % Load the USK library for SmartDongle functions.
    loadlibrary('USK', 'usk.h');
    
    % Call LedRed from the USK library.
    calllib('USK','LedRed');
    
    % Create a c-style pointer to a uint8 object containing blanks equal to
    % the number of bytes to be read from the SmartDongle.
    Buffer = libpointer('uint8Ptr', unicode2native(blanks(length(WriteData)), 'US-ASCII'));
    
    % Call the Read function from the USK library to read the data from the
    % SmartDongle into Buffer. Pass P1, P2, memory address to read from,
    % pointer to the byte array to read into, and the number of bytes to be
    % read from the SmartDongle. Returns an error code and Buffer
    % containing the data read from the SmartDongle.
    [error, Buffer] = calllib('USK', 'Read', P1, P2, address, Buffer, length(WriteData));
    
    % An error code of zero indicates a success, all other numbers are
    % errors.
    if(error ~= 0)
        % Create a pointer to a c-style string with 128 characters to allow
        % space for the max error message length.
        ErrMsg = libpointer('cstring', blanks(128));
        
        % Call GetErrorString from the USK library passing in the c-style
        % string ErrMsg and the error code from the previous Read call. The
        % error message will be returned into ErrMsg.
        [ErrMsg] = calllib('USK', 'GetErrorString', ErrMsg, error);
        
        % Print the error message.
        fprintf('\n%s\n\n',ErrMsg)
    else
        % Since all data read from the SmartDongle is in byte array form,
        % conver the data read back to a string for comparisons.
        Buffer = native2unicode(Buffer, 'US-ASCII');
        
        % Compare the data read to that written in example2a.
        if(strcmp(Buffer, WriteData) == 1)
            % Match, success
            fprintf('\nSuccess: Found Valid Application Data\n\n')
        else
            % No match, fail
            fprintf('\nFailed: Invalid Application Data\n\n')
        end
    end
    
    % Call LedGreen from the USK library.
    calllib('USK', 'LedGreen');
    
    % Unload the USK library.
    unloadlibrary('USK');
end

