%    example3b.m:  Example that read and decrypts a user string from
%                  a SmartDongle as written in example3a.c

%    www.smartdongle.com

%    MicroWorks, Inc.
%    2808 North Cole Road
%    Boise, ID 83704
%    (208)-375-1234

function example3b()
    % Your personal keys go here.
    P1 = hex2uint64('EC6CC589AEFD1E75');
    P2 = hex2uint64('FCEC0A6A82747B3F');
    
    % User space memory addres to read from the SmartDongle.
    address = 0;
    
    % Unencrypted data written to the SmartDongle in example3a. Used here
    % to compare and verify what read from the SmartDongle and decrypted
    % matches what was encrypted and written to the SmartDongle in
    % example3a.
    WriteSecret = 'This is the secret written to SmartDongle, it can be passwords, serial numbers, program parameters, etc';
    
    % This is the key that will be used to encrypt your data. This key must
    % match the one used to encrypt the data.
    EncryptionKey = '7obdgEn2{TH>]SE|';
    
    % The decryption function requires that the key be in byte array form
    % so this function will convert the string to a byte array.
    keyBytes = unicode2native(EncryptionKey, 'US-ASCII');
    
    % Load the USK library for SmartDongle functions.
    loadlibrary('USK', 'usk.h');
    
    % Call LedRed from the USK library.
    calllib('USK','LedRed');
    
    % Create a c-style pointer to a uint8 object (byte array) to store the
    % encrypted data read from the SmartDongle. The size of the object must
    % match the size of the encrypted data written to the SmartDongle and
    % therefore will be a multiple of 16.
    Buffer = libpointer('uint8Ptr', uint8((zeros(1, 16 * (ceil(length(WriteSecret)/16))))));
    
    % Call the Read function from the USK library. (See example2b for a
    % description of the Read function)
    [error, Buffer] = calllib('USK', 'Read', P1, P2, address, Buffer, 16*(ceil(length(WriteSecret)/16)));
    
    % An error code of zero indicates a success, all other numbers are
    % errors.
    if(error ~= 0)
        % Create a pointer to a c-style string with 128 characters to allow
        % space for the max error message length.
        ErrMsg = libpointer('cstring', blanks(128));
        
        % Call GetErrorString from the USK library passing in the c-style
        % string ErrMsg and the error code from the previous Read call. The
        % error message will be returned into ErrMsg.
        [ErrMsg] = calllib('USK', 'GetErrorString', ErrMsg, error);
        
        % Print the error message.
        fprintf('\n%s\n\n',ErrMsg)
        
        % Call LedGreen from the USK library.
        calllib('USK', 'LedGreen');
        
        % Unload the USK library.
        unloadlibrary('USK');
        return
    else
        % Create a c-style pointer to the encrypted data read from the
        % SmartDongle.
        secretPtr = libpointer('uint8Ptr', Buffer);
        
        % Create a c-style pointer to the decryption key byte array.
        keyPtr = libpointer('uint8Ptr', keyBytes);
        
        % Call the FileDecrypt function from the USK library. Pass the
        % pointer to the encrypted data, the number of blocks to be
        % encrypted (this value is aligned to 16 byte blocks), the pointer
        % to the decryption key, and the block size in bytes (always 128).
        % Returns the decrypted data and the original decryption key.
        [secretPtr, keyPtr] = calllib('USK', 'FileDecrypt', secretPtr, ceil(length(WriteSecret)/16), keyPtr, 128);
        
        % Convert the decrypted byte array data back to a string to be
        % compared with the original message.
        DecryptedSecret = native2unicode(secretPtr, 'US-ASCII');
        
        % Compare the decrypted data read from the SmartDongle with the
        % original.
        if(strcmp(deblank(DecryptedSecret), WriteSecret) == 1)
            % Match, success
            fprintf('\nSuccess: Your secret has been verified\n\n')
        else
            % No match, fail
            fprintf('\nFailure: decoded data does not match your secret\n\n')
        end
    end

     % Call LedGreen from the USK library.
    calllib('USK', 'LedGreen');

    % Unload the USK library.
    unloadlibrary('USK');
end

