/*
   usk.h for SmartDongle API library version 3.2


   www.smartdongle.com

   MicroWorks, Inc.
   2808 North Cole Road
   Boise, ID 83704
*/

#ifndef uint8
#define uint8  unsigned char
#endif

#ifndef uint32
#define uint32 unsigned long int
#endif

#ifndef _DEFINED_ED8943FD_17C0_4FEB_A743_5E50B68C6A6E
#define _DEFINED_ED8943FD_17C0_4FEB_A743_5E50B68C6A6E


/*
   Three on-chip SmartDongle encrpytion modes to choose from.
   This provide protection for SmartDongle on-board memory only.
   Select QTS_OPTION_CRYPT_IO to encrypt data written to SmartDongle
   user memory.
*/
#define QTS_OPTION_ON_CHIP_EPROM_PROTECT_STRONG   0x00
#define QTS_OPTION_ON_CHIP_EPROM_PROTECT_DISABLE  0x01
#define QTS_OPTION_ON_CHIP_EPROM_PROTECT_FAST     0x02

/*
   Maximum string length returned from GetErrorString()
*/
#define QTS_ERROR_STRING_LENGTH_MAX 128


/* Same structure as Shell DLLVersionInfo in shlwapi.h */
typedef struct _USKDllVersionInfo
{
   DWORD cbSize;
   DWORD dwMajorVersion;                   /* Major version */
   DWORD dwMinorVersion;                   /* Minor version */
   DWORD dwBuildNumber;                    /* Build number */
   DWORD dwPlatformID;                     /* DLLVER_PLATFORM_* */
} USKDLLVERSIONINFO;

/* Platform IDs for DLLVERSIONINFO */
#define EXPORT_DLL_FUNCTIONVER_PLATFORM_WINDOWS         0x00000001      /* Windows 95 */
#define EXPORT_DLL_FUNCTIONVER_PLATFORM_NT              0x00000002      /* Windows NT */

   #ifndef EXPORT_DLL_FUNCTION
   /*
      Cause following DLL functions as to be defined as imports, unless
      DLL.h has overridden this behavior.
   */
      #define EXPORT_DLL_FUNCTION(type) type __stdcall
   #endif


/*
   Given the correct keys, P1 and P2, will place the dongle into
   the enabled state and read EEPROM into buffer.

   Valid address is 0 to 32431

   Note: Refer to sd_uskerr.h for returned errors.
*/
   EXPORT_DLL_FUNCTION(int) Read(
      unsigned __int64 P1,
      unsigned __int64 P2,
      int address,
      uint8 *buffer,
      int size);

/*
   Given the correct keys, P1 and P2, will place the dongle into
   the enabled state and buffer into EEPROM.

   Valid address is 0 to 32431

   Note: Refer to sd_uskerr.h for returned errors.
*/
   EXPORT_DLL_FUNCTION(int) Write(
      unsigned __int64 P1,
      unsigned __int64 P2,
      int address,
      uint8 *buffer,
      int size);


/*
   LED Control.
*/
   EXPORT_DLL_FUNCTION(int) LedGreen();
   EXPORT_DLL_FUNCTION(int) LedRed();
   EXPORT_DLL_FUNCTION(int) LedOff();

/*
   User must allocate CP2_SERIALNUM_LENGTH+1 sn buffer
*/
   EXPORT_DLL_FUNCTION(int) GetSerialNumber(char *sn);


/*
   Return error description string corresponding to error code.
   User must allocate string size of QTS_ERROR_STRING_LENGTH_MAX
*/
	EXPORT_DLL_FUNCTION(void) GetErrorString(
		char *errorString, int error);

    EXPORT_DLL_FUNCTION(void) FileDecrypt(uint8 *scrt, uint32 scrtLen, uint8 *ky, uint32 blockLen);
    EXPORT_DLL_FUNCTION(void) FileEncrypt(uint8 *scrt, uint32 scrtLen, uint8 *ky, uint32 blockLen, uint8 *outBuffer);
   
#endif /* _DEFINED_ED8943FD_17C0_4FEB_A743_5E50B68C6A6E */
